<?php

class Centralpay3dsModuleFrontController extends ModuleFrontController {

  public $module;

  const VIEWS_TEMPALTES_FRONT = '/views/templates/front/';

  const ACSURL = 'acsURL';

  const PAREQ = 'paReq';

  public function __construct() {
    $this->controller_type = 'modulefront'; //instead of standard controllers’ front

    $this->module = Module::getInstanceByName(Tools::getValue('module')); // an instance of the module is created
    if (!$this->module->active) {
      Tools::redirect('index'); //if the module is inactive, the user will be redirected to index.php
    }
    $this->page_name = 'module-' . $this->module->name . '-' . Dispatcher::getInstance()
        ->getController(); //for example: module-belvg_storelocator-search

    parent::__construct();
  }


  /**
   * Assign module template
   *
   * @param string $template
   */
  public function setTemplate($template) {
    if (Tools::file_exists_cache(_PS_THEME_DIR_ . 'modules/' . $this->module->name . '/' . $template)) {
      $this->template = _PS_THEME_DIR_ . 'modules/' . $this->module->name . '/' . $template; //if the file with the template is located in the root directory
    }
    elseif (Tools::file_exists_cache($this->getTemplatePath($template))) {
      $this->template = $this->getTemplatePath($template); //otherwise file in  module_dir/views/templates/front/ should be used
    }
    else {
      throw new PrestaShopException("Template '$template'' not found"); //if the file is not found, the exception will be thrown
    }
  }

  /**
   * Get path to front office templates for the module
   *
   * @return string
   */
  public function getTemplatePath($template) {
    return _PS_MODULE_DIR_ . $this->module->name . self::VIEWS_TEMPALTES_FRONT . $template;
  }

  public function postProcess() {


  }

  /**
   * Assign template vars related to page content
   *
   * @see FrontController::initContent()
   */
  public function initContent() {

    $this->disableColumns();
    parent::initContent();

    $retour = json_decode($this->context->cookie->retour3DS, 1);
    if (!isset($retour[self::ACSURL])) {
      $condition1 = TRUE;
    }
    else {
      $condition1 = FALSE;
    }
    if (isset($retour[self::ACSURL]) && $retour[self::ACSURL] == '') {
      $condition2 = TRUE;
    }
    else {
      $condition2 = FALSE;
    }

    if (isset($retour[self::PAREQ]) && $retour[self::ACSURL] == '') {
      $condition3 = TRUE;
    }
    else {
      $condition3 = FALSE;
    }

    if ($condition1 || !isset($retour[self::PAREQ]) || $condition2 || $condition3) {

      //on transforme le panier en commande erreur de paiement
      $link = $this->errorCommande();

      Tools::redirect($link);
      exit();
    }


    $this->context->smarty->assign(
      [
        self::PAREQ => $retour[self::PAREQ],
        self::ACSURL => $retour[self::ACSURL],
        'merchant_description' => $retour['merchantName'],
        'retour' => $this->context->link->getModuleLink('centralpay', '3dsretour'),

      ]
    );
    return $this->setTemplate('3ds.tpl');

  }


  protected function getModuleDir($data = NULL) {
    $path = _PS_MODULE_DIR_ . $this->module->name . self::VIEWS_TEMPALTES_FRONT;
    if (!is_null($data)) {
      return _PS_MODULE_DIR_ . $this->module->name . self::VIEWS_TEMPALTES_FRONT . $data . '/';
    }

    if (!is_null($data)) {
      throw new PrestaShopException('Template not mapped');
    }

    return $path;
  }

  protected function disableColumns() {
    $this->display_column_left = FALSE;
    $this->display_column_right = FALSE;
    $this->display_footer = FALSE;
    $this->display_header = FALSE;
  }

  public function errorCommande() {

    $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'module WHERE name=\'' . pSQL('centralpay') . '\'';
    $results = Db::getInstance()->ExecuteS($sql);

    $id_module = $results[0]["id_module"];


    $panier = $this->context->cart;
    $cart = new Cart((int) $panier->id);
    $order_status = (int) Configuration::get('PS_OS_ERROR');
    $order_total = $cart->getOrderTotal(TRUE, Cart::BOTH);
    $this->module->validateOrder($panier->id, $order_status, $order_total, $this->module->l('Credit/Debit card', $this->module->name), NULL, [], NULL, FALSE, $panier->secure_key);
    $orderid = Order::getOrderByCartId((int) ($cart->id));

    $order = new Order($orderid);
    return 'index.php?controller=order-confirmation&id_cart=' . $order->id_cart
      . '&id_module=' . $id_module . '&id_order=' . $order->id
      . '&key=' . $order->secure_key . '&error=yes';

  }

}