<?php

class Centralpay3dsretourModuleFrontController extends ModuleFrontController {

  public $module;

  const CENTRALPAY = '_CENTRALPAY_';

  const RETOUR_3DS = 'retour3DS';

  const CARD_ID = 'cardId';

  const PARES = 'PaRes';

  const CUSTOMER_ID = 'customerId';

  const VIEWS_TEMPLATE_FRONT = '/views/templates/front/';

  const NOM_MODULE = 'centralpay';

  const AMOUNT = 'amount';

  public function __construct() {
    $this->controller_type = 'modulefront'; //instead of standard controllers’ front

    $this->module = Module::getInstanceByName(Tools::getValue('module')); // an instance of the module is created
    if (!$this->module->active) {
      Tools::redirect('index'); //if the module is inactive, the user will be redirected to index.php
    }
    $this->page_name = 'module-' . $this->module->name . '-' . Dispatcher::getInstance()
        ->getController(); //for example: module-belvg_storelocator-search


    parent::__construct();
  }


  /**
   * Assign module template
   *
   * @param string $template
   */
  public function setTemplate($template) {
    if (Tools::file_exists_cache(_PS_THEME_DIR_ . 'modules/' . $this->module->name . '/' . $template)) {
      $this->template = _PS_THEME_DIR_ . 'modules/' . $this->module->name . '/' . $template; //if the file with the template is located in the root directory
    }
    elseif (Tools::file_exists_cache($this->getTemplatePath($template))) {
      $this->template = $this->getTemplatePath($template); //otherwise file in  module_dir/views/templates/front/ should be used
    }
    else {
      throw new PrestaShopException("Template '$template'' not found"); //if the file is not found, the exception will be thrown

    }
  }

  /**
   * Get path to front office templates for the module
   *
   * @return string
   */
  public function getTemplatePath($template) {
    return _PS_MODULE_DIR_ . $this->module->name . self::VIEWS_TEMPLATE_FRONT . $template;
  }

  public function postProcess() {

  }

  /**
   * Assign template vars related to page content
   *
   * @see FrontController::initContent()
   */
  public function initContent() {
    $this->disableColumns();
    parent::initContent();

    if (Configuration::get(self::CENTRALPAY . 'MODE_TEST') == 1) {
      $url_api = API_TEST;
    }
    else {
      $url_api = API;

    }
    $panier = $this->context->cart;
    $currency = new Currency($panier->id_currency);
    // IDENTIFICATION API CENTRALPAY
    $userpwd = Configuration::get(self::CENTRALPAY . 'LOGIN') . ":" . Configuration::get(self::CENTRALPAY . 'PASSWORD');
    // ADRESSE IP DU CLIENT
    if (Configuration::get(self::CENTRALPAY . 'MODE_TEST') == 1) {
      //si on est en mode de test on récupère l'adresse IP renseigné dans le BO
      $adresse_ip = Configuration::get(self::CENTRALPAY . 'IP');
    }
    else {
      $adresse_ip = $_SERVER["REMOTE_ADDR"];
    }
    // Récupération des données en session et des valeurs postées
    $session = json_decode($this->context->cookie->retour3DS, 1);
    // Si le paramétre PaRes est vide on considère une erreur de paiement
    if (empty(Tools::getValue(self::PARES))) {
      $this->context->cookie->__unset(self::RETOUR_3DS);
      $this->context->cookie->__unset(self::CARD_ID);
      $this->context->cookie->__unset(self::PARES);
      //on transforme le panier en commande erreur de paiement
      $link = $this->errorCommande();
      $this->context->smarty->assign(
        [
          'url' => $link,
          self::PARES => '',

        ]
      );
      return $this->setTemplate('3dsredirect.tpl');

    }

    // Récupération du customer id

    $sql = 'SELECT ' . self::CUSTOMER_ID . ' FROM ' . _DB_PREFIX_ . 'centralpay_customer WHERE cmsCustomerId=' . intval($panier->id_customer);
    $results = Db::getInstance()->ExecuteS($sql);
    $customerid = $results[0][self::CUSTOMER_ID];
    $trans['currency'] = $currency->iso_code;
    $trans[self::AMOUNT] = intval($session[self::AMOUNT]);
    $trans[self::CUSTOMER_ID] = $customerid;
    $trans[self::CARD_ID] = $this->context->cookie->cardId;
    $trans['merchantTransactionId'] = $panier->id;
    $trans['pointOfSaleId'] = Configuration::get(self::CENTRALPAY . 'POINTOFSALEID');
    $trans['endUserIp'] = $adresse_ip;
    $trans["3ds"] = [
      "enrollmentId" => $session["enrollmentId"],
      "paRes" => Tools::getValue(self::PARES),
    ];

    if (isset($this->context->cookie->opt) && $this->context->cookie->opt == 1) {

      $otp[self::CUSTOMER_ID] = $customerid;
      $otp['provider'] = Configuration::get(self::CENTRALPAY . 'PROVIDER');
      $otp['currency'] = $currency->iso_code;
      $otp['amount'] = intval($panier->getordertotal(TRUE) * 100);
      $otpfields = isset($otp) ? http_build_query($otp) : null;
      $curl = curl_init();
      curl_setopt($curl, CURLOPT_URL, trim($url_api) . "customer/" . $customerid . "/otp");
      curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($curl, CURLOPT_POSTFIELDS, $otpfields);
      curl_setopt($curl, CURLOPT_POST, 1);
      curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($curl, CURLOPT_USERPWD, $userpwd);
      $result = curl_exec($curl);
      $info = curl_getinfo($curl);
      $retour = json_decode($result, 1);
      curl_close($curl);
      $link = $this->context->link->getModuleLink(self::NOM_MODULE, 'otp');

      $this->context->smarty->assign(
        [
          'url' => $link,
          self::PARES => Tools::getValue(self::PARES),

        ]
      );
      return $this->setTemplate('3dsredirect.tpl');

    }

    if (isset($this->context->cookie->cvc) && $this->context->cookie->cvc == 1) {
      $link = $this->context->link->getModuleLink(self::NOM_MODULE, 'cvc');
      $this->context->smarty->assign(
        [
          'url' => $link,
          self::PARES => Tools::getValue(self::PARES),

        ]
      );
      return $this->setTemplate('3dsredirect.tpl');
    }


    $transfields = isset($trans) ? http_build_query($trans) : null;
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url_api . "transaction");
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $transfields);
    curl_setopt($curl, CURLOPT_POST, 1);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
    curl_setopt($curl, CURLOPT_USERPWD, $userpwd);
    $result = curl_exec($curl);
    $info = curl_getinfo($curl);
    $retour = json_decode($result, 1);
    curl_close($curl);
    //Vérification de l'état de la transaction
    if (isset($info["http_code"]) && $info["http_code"] == 200 && isset($retour["authorizationStatus"]) && $retour["authorizationStatus"] == 'SUCCESS') {

      $sql = 'SELECT id_module FROM ' . _DB_PREFIX_ . 'module WHERE name=\'' . pSQL(self::NOM_MODULE) . '\'';
      $results = Db::getInstance()->ExecuteS($sql);
      $id_module = $results[0]["id_module"];
      //transaction ok on transforme le panier en commande validée et payée
      $cart = new Cart((int) $panier->id);
      $order_status = (int) Configuration::get('PS_OS_PAYMENT');
      $order_total = $cart->getOrderTotal(TRUE, Cart::BOTH);
      $this->module->validateOrder($panier->id, $order_status, $order_total, $this->module->l('Credit/Debit card', $this->module->name), NULL, ['transaction_id' => $retour["transactionId"]], NULL, FALSE, $panier->secure_key);

      //Enregistrement de la liaison entre le paiement et la carte centralpay
      $orderid = Order::getOrderByCartId((int) ($cart->id));
      $order = new Order($orderid);
      $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'order_payment WHERE order_reference=\'' . pSQL($order->reference) . '\'';
      $results = Db::getInstance()->ExecuteS($sql);
      if (is_array($results)) {
        foreach ($results as $r) {
          if ($r["transaction_id"] == $retour["transactionId"] && Tools::getValue("saveCard") == 1) {

            //Enregistrement du lient entre la carte et le paiement Prestashop si la carte a été sauvegardée

            Db::getInstance()->insert('centralpay_payment_card', [
              self::CARD_ID => pSQL($this->context->cookie->cardId),
              'cmsPaymentId' => pSQL($r["id_order_payment"]),
            ]);
          }
        }
      }
      $this->context->cookie->__unset(self::RETOUR_3DS);
      $this->context->cookie->__unset(self::CARD_ID);
      $this->context->cookie->__unset('customerValidationValue');

      $link = '/confirmation-commande?id_cart=' . $order->id_cart
        . '&id_module=' . $id_module . '&id_order=' . $order->id
        . '&key=' . $order->secure_key;
    }
    else {
      $this->context->cookie->__unset(self::RETOUR_3DS);
      $this->context->cookie->__unset(self::CARD_ID);
      //on transforme le panier en commande erreur de paiement
      $link = $this->errorCommande();
    }
    $this->context->smarty->assign(
      [
        'url' => $link,
      ]
    );
    return $this->setTemplate('3dsredirect.tpl');
  }


  protected function getModuleDir($data = NULL) {
    $path = _PS_MODULE_DIR_ . $this->module->name . self::VIEWS_TEMPLATE_FRONT;
    if (!is_null($data)) {
      return _PS_MODULE_DIR_ . $this->module->name . self::VIEWS_TEMPLATE_FRONT . $data . '/';
    }

    if (!is_null($data)) {
      throw new PrestaShopException('Template not mapped');
    }
    return $path;
  }

  protected function disableColumns() {
    $this->display_column_left = FALSE;
    $this->display_column_right = FALSE;
    $this->display_footer = FALSE;
    $this->display_header = FALSE;
  }

  public function errorCommande() {

    $sql = 'SELECT * FROM ' . _DB_PREFIX_ . 'module WHERE name=\'' . pSQL(self::NOM_MODULE) . '\'';
    $results = Db::getInstance()->ExecuteS($sql);

    $id_module = $results[0]["id_module"];


    $panier = $this->context->cart;
    $cart = new Cart((int) $panier->id);
    $order_status = (int) Configuration::get('PS_OS_ERROR');
    $order_total = $cart->getOrderTotal(TRUE, Cart::BOTH);
    $this->module->validateOrder($panier->id, $order_status, $order_total, $this->module->l('Credit/Debit card', $this->module->name), NULL, [], NULL, FALSE, $panier->secure_key);
    $orderid = Order::getOrderByCartId((int) ($cart->id));

    $order = new Order($orderid);
    return '/confirmation-commande?id_cart=' . $order->id_cart
      . '&id_module=' . $id_module . '&id_order=' . $order->id
      . '&key=' . $order->secure_key . '&error=yes';


  }

}